import click


@click.command('test')
@click.pass_context
def cli(ctx):
    """Run tests for your application (pytest)"""
    click.echo("test command running!")
    ctx.obj.run_in_docker('pytest', {'PALM_TEST': True})



from typing import Optional, Tuple
import importlib
from pathlib import Path
from typing import Optional, List, Tuple
import click
from palm.utils import run_on_host
from palm.plugin_manager import PluginManager
from .palm_config import PalmConfig
from .code_generator import CodeGenerator


class Environment:
    def __init__(self, plugin_manager: PluginManager, palm_config: PalmConfig):
        self.home = Path.cwd()
        self.palm = palm_config
        self.plugin_manager = plugin_manager

    def run_in_docker(
        self, cmd: str, env_vars: Optional[dict] = {}
    ) -> Tuple[bool, str]:
        """Shells out and runs the cmd in docker

        Args:
            cmd (str): The command you want to run
            env_vars (Optional[dict], optional): Dict of env vars to pass to the docker container.
        """
        click.secho(f"Executing command `{cmd}` in compose...", fg="yellow")

        docker_cmd = ['docker-compose run --service-ports --rm']
        docker_cmd.extend(self._build_env_vars(env_vars))
        docker_cmd.append(self.palm.image_name)
        docker_cmd.append(f'/bin/bash -c "{cmd}" ')

        ex_code, _, _ = run_on_host(' '.join(docker_cmd))
        if ex_code == 0:
            return (True, 'Success! Palm completed with exit code 0')
        return (False, f"Fail! Palm exited with code {ex_code}")

    def run_in_shell(self, cmd: str, env_vars: Optional[dict] = {}):
        """deprecated - use run_in_docker"""

        deprecation_msg = (
            "DEPRECATION: run_in_shell has been renamed to "
            "`run_in_docker` and will be removed in a future version. "
            "Please update your commands to use ctx.obj.run_in_docker"
        )
        click.secho(deprecation_msg, fg="yellow")
        success, msg = self.run_in_docker(cmd, env_vars)
        click.secho(msg, fg="green" if success else "red")

    def run_on_host(
        self,
        cmd: str,
        check: Optional[bool] = False,
        capture_output: Optional[bool] = False,
    ) -> Tuple[int, str, str]:
        """context wrapper for :obj:`palm.utils.run_on_host`"""
        return run_on_host(cmd, check, capture_output)

    def import_module(self, module_name: str, module_path: Path):
        """Imports a module from a path

        This is useful if you need to import a python module to a command
        defined in your local project, unfortunately normal relative `import`
        statements don't work in that context because palm is running in your
        python path, so you need to use this instead.

        Args:
            module_name (str): Name of the module
            module_path (Path): Absolute Path to the module file

        Returns:
            ModuleSpec: The module spec of the imported module
        """
        try:
            spec = importlib.util.spec_from_file_location(module_name, module_path)
            mod = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(mod)
        except ImportError as error:
            click.secho(f'Import error: {error}', fg="red")
            return
        return mod

    def generate(
        self, template_path: Path, target_path: Path, replacements: dict
    ) -> str:
        return CodeGenerator(template_path, target_path, replacements).run()

    def _build_env_vars(self, env_vars: dict) -> List[str]:
        env_vars_list = []
        for key in env_vars.keys():
            env_vars_list.append(f'-e {key.upper()}={env_vars[key]}')
        return env_vars_list



from pathlib import Path
from typing import List, Optional
import yaml
import shutil
import fileinput


def create_config(
    palm_dir,
    image_name,
    plugins: Optional[List] = [],
    protected_branches: Optional[List] = [],
):
    config_path = f'{palm_dir}/config.yaml'
    base_config = {
        'image_name': image_name,
        'plugins': plugins,
        'protected_branches': protected_branches,
    }

    with open(config_path, 'w') as file:
        yaml.safe_dump(base_config, file)


def create_command(ctx, command: str, template_dir: Path, target_dir: Path):
    """Creates a new cmd file from template"""

    replacements = {
        'command': command,
    }

    ctx.obj.generate(template_dir, target_dir, replacements)



import click


@click.group()
def cli():
    """{{group}}"""
    pass


{% for command in commands %}
@cli.command()
@click.pass_context
def {{command}}(ctx):
    """{{command}}"""
    click.echo("{{command}} command running!")

{% endfor %}


from pathlib import Path
from palm.plugins.base import BasePlugin

# This plugin is intended for use in testing palm-cli.
# It is not intended to be used in production.

TestInternalPlugin = BasePlugin(
    name='test_internal',
    command_dir=Path(__file__).parent / 'commands',
)



from pathlib import Path
from typing import Optional, List
from pygit2 import Repository
from click import secho
import yaml


class PalmConfig:
    """Palm config class
    Reads the .palm/config.yaml from the current project
    Makes config available to other modules

    Args:
        project_root: The root path object if not cwd
    """

    branch: str = None

    def __init__(self, project_path: Optional["Path"] = Path.cwd()):
        self.project_root = project_path
        self.config = self._get_config()
        self.branch = self._get_current_branch()

    def _get_current_branch(self) -> str:
        return Repository(str(self.project_root)).head.shorthand

    def _get_config(self) -> object:
        config_path = self.project_root / '.palm' / 'config.yaml'
        if not config_path.exists():
            secho(
                'No palm config found in .palm/config.yml, please run \'palm scaffold config\'',
                fg='yellow',
            )
            secho(
                'Some palm commands may not work correctly without palm config',
                fg='yellow',
            )
            return {}

        return yaml.safe_load(config_path.read_text())

    def validate_branch(self) -> None:
        """Raises SystemExit if branch is protected."""
        if self.branch not in self.protected_branches:
            return
        msg = f"You are currently on protected branch {self.branch}. For your safety Palm will not run!"
        secho(msg, fg="red")
        raise SystemExit(msg)

    @property
    def has_config(self) -> bool:
        return len(self.config.keys()) > 0

    @property
    def protected_branches(self) -> List[Optional[str]]:
        """Returns the list of configured protected branches for the current repo

        Returns:
             list[Optional[str]]: list of branch names e.g ['main', 'master']
        """
        return self.config.get('protected_branches') or []

    @property
    def project_root_snake_case(self):
        return self.project_root.name.replace('-', '_')

    @property
    def image_name(self) -> str:
        """Docker image name for the current project
        Attempts to load the image_name from .palm/config.yaml, falling back to
        the snake_cased project root dir name

        Returns:
            str: Name of docker image to use
        """
        return self.config.get('image_name') or self.project_root_snake_case

    @property
    def plugins(self) -> list:
        core_plugins = ['core']
        plugins_from_config = self.config.get('plugins') or []
        # The order here defines the order in which commands will be overridden
        # Plugins on the right will override plugins on the left!
        return core_plugins + plugins_from_config + ['repo']
